//----------------------------------------------------------------------------------
// File:   nv_ISvcUI.h 
// Email:  tlorach@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------
#ifndef __IDlgSettings__
#define __IDlgSettings__

#define WINDOWS_VERSION "3.04"

#include "nv_ISvcBase.h"

using namespace nvSvc;

class IEventsWnd;
class IWindowContainer;
class IWindow;
class IWindowConsole;
class IWindowHandler;
class IWindowLog;
class IWindowSplitter;
class IControlCombo;
class IControlListBox;
class IControlScalar;
class IControlString;
class IControlToolbar;
class IControlVector;
/*************************************************************************/ /**
 ** Common Window definition
 **/ /*************************************************************************/
class IWindow : public ISmartRef
{
public:
	DECLSMARTREF(IWindow); ///< Macro for GetRef(), AddRef(), Release() , QueryInterface() implementations
	virtual IWindow* PeekMyself() = 0; ///< We should remove this...
	virtual IWindow* OnRender(float fElapsedTime) = 0; ///< in the case where the UI is rendered with 3D, for example
	virtual bool MsftMsgProc( unsigned long hWnd, unsigned int uMsg, unsigned int wParam, unsigned long lParam ) = 0; ///< specific to Microsoft msg pump

	virtual IWindow* Minimize(int bYes=1) = 0;
	virtual void *GetCWnd() = 0; // void* because we don't want MS here...
	virtual unsigned long GetHandle() = 0; // unsigned long because we don't want MS here (hwnd)...

    virtual IWindow* Register(IEventsWnd *pDataBase) = 0;
	virtual IWindow* UnRegister(IEventsWnd *pClient=NULL) = 0;

    /*virtual bool CanHaveFocus() = 0;
    virtual void OnFocusIn() = 0;
    virtual void OnFocusOut() = 0;
    virtual void OnMouseEnter() = 0;
    virtual void OnMouseLeave() = 0;
    virtual void OnHotkey() = 0;*/

    virtual IWindow* SetEnabled( bool bEnabled ) = 0;
    virtual bool GetEnabled() = 0;
    virtual IWindow* SetVisible(int bYes=1) = 0;
	virtual int GetVisible() = 0; ///< bYes=-1 to query the state

    virtual unsigned int        GetType() = 0;

    virtual LPCSTR              GetID() const = 0;
    virtual IWindow* SetID( LPCSTR ID ) = 0;

    virtual IWindow* SetLocation(int x, int y) = 0;
    virtual IWindow* SetSize(int width, int height) = 0;
    virtual IWindow* GetPosition(int *x, int *y, int *width, int *height) = 0;

    virtual IWindow* SetHotkey( UINT nHotkey ) = 0;
    virtual UINT GetHotkey() = 0;
    virtual IWindow* SetUserData( void *pUserData ) = 0;
    virtual void *GetUserData() const = 0;

	virtual IWindow* SetZPos(int z=1) = 0;
	virtual IWindow* SetTitle(LPCSTR title) = 0;
	virtual IWindow* GetTitle(char * title, int maxlen) = 0;

	virtual IWindow* SetBGColor(float r, float g, float b, float a) = 0;
	virtual IWindow* SetBorderColor(float r, float g, float b) = 0;
	virtual IWindow* EnableBG(bool bYes) = 0;
	virtual IWindow* EnableBorders(bool bYes) = 0;
	//
	// links with parent (new from v1.37)
	//
	virtual IWindowContainer *GetParentContainer() = 0;
	virtual IWindow* SetParentContainer(IWindowContainer *pWin) = 0;
	//
	// To destry the window. Note: this instance may still be referenced to various objects (RefCnt > 0).
	// so the destroy status will release some resources and will set as invalid.
	// Note that Destroy will notify registered users with the event 'rtDestroyingWindow', so this is the place where the user
	// can release the references to it.
	// (new from v1.37)
	//
	virtual IWindow* Destroy() = 0;
	virtual bool IsValid() = 0; // false is this was destroyed. This can be used to release the reference to this window
    //
    // Cast
	//
    virtual IWindowContainer    *AsWindowContainer() = 0;
    virtual IWindowConsole      *AsWindowConsole() = 0;
    virtual IWindowHandler      *AsWindowHandler() = 0;
    virtual IWindowLog          *AsWindowLog() = 0;
    virtual IWindowSplitter     *AsWindowSplitter() = 0;
    virtual IControlCombo       *AsControlCombo() = 0;
    virtual IControlListBox     *AsControlListBox() = 0;
    virtual IControlScalar      *AsControlScalar() = 0;
    virtual IControlString      *AsControlString() = 0;
    virtual IControlToolbar     *AsControlToolbar() = 0;
    virtual IControlVector      *AsControlVector() = 0;
};


/*************************************************************************/ /**
 ** 
 ** must be implemented by the client if it wants to get back events
 ** 
 ** \u Note : if return false, it means that the object may handle this 
 ** event with a default process
 **/ /*************************************************************************/
class IEventsWnd
{
public:
    virtual void MouseEvent(IWindow *pWin, float mx, float my, unsigned int buttons) {};
    virtual void ComboSelectionChanged(IControlCombo *pWin, unsigned int selectedidx) {};
    //virtual void RadioSelectionChanged(IControlScalar *pWin, unsigned int selectedidx) {};
    virtual void ListBoxSelection(IControlListBox *pWin, int selectstart, int selectend) {};
    virtual void Button(IWindow *pWin, int pressed) {};
    virtual void ColorPicker(IWindow *pWin, float *color, float *prev) {};
    virtual void VectorChanged(IControlVector *pWin, float *v, float *prev, int dim) {};
    virtual void ScalarChanged(IControlScalar *pWin, float v, float prev) {};
    virtual void CheckBoxChanged(IControlScalar *pWin, bool value, bool prev) {};
    virtual void ToolbarChanged(IControlToolbar *pWin, int selecteditem, int prev) {};
    virtual void StringChanged(IControlString *pWin) {};
    virtual void ConsoleChanged(IWindowConsole *pWin, LPCSTR newstring, LPCSTR prev) {};

    virtual bool WindowClosed(IWindow *) { return false; };
	virtual bool DestroyingWindow(IWindow *) { return false; }; ///< called when we asked for this window to be destroyed (1.37).
	virtual bool WindowResized(IWindow *, int cx, int cy) { return false; };
	virtual bool WindowRepaint(IWindow *) { return false; };
};

class IProgressBar : public IWindow
{
public:
	virtual IProgressBar* SetPercent(float l) = 0;
	virtual IProgressBar* AddPercent(float l) = 0;
	virtual IProgressBar* SetMessage(LPCSTR lpstr) = 0;
};

/*************************************************************************/ /**
 **
 ** Static control : Just display text
 ** 
 **/ /*************************************************************************/
class IControlStatic : public IWindow
{
public:
    virtual bool GetTextCopy( LPSTR strDest, UINT bufferCount ) = 0;
    virtual LPCSTR GetText()= 0;
    virtual IControlStatic* SetText( LPCSTR strText ) = 0;
};
/*************************************************************************/ /**
 ** Logging dlg box interface
 **
 ** various cases in the line:
 ** <LI> no header : common msg
 ** <LI> "<<warning>> message" : Warning msg
 ** <LI> "<<error>> message" : Error Msg
 ** <LI> "<<note>> message" : event Msg used by entities
 ** <LI> "<<distrib>> message" : Networking msg
 ** <LI> "<<no>> message" : negative Msg
 ** <LI> "<<event>> message" : event Msg
 ** <LI> "<<yes>> message" : yes Msg
 **/ /*************************************************************************/ 
class IWindowLog : public IWindow
{
public:
	virtual IWindowLog* Clear() = 0;
	virtual IWindowLog* AddMessage(LPCSTR fmt, ...) = 0;
	virtual IWindowLog* AddMessage(int level, LPCSTR fmt, ...) = 0;
};


/*************************************************************************/ /**
 ** Cardiogram-like graphic.
 ** \todo create methods to display this timeline...
 **/ /*************************************************************************/ 
class IWindowConsole : public IWindow
{
public:
	virtual IWindowConsole* SetItalic(bool bYes=true) = 0;
	virtual IWindowConsole* SetBold(int val=1) = 0;
	virtual IWindowConsole* SetUnderline(bool bYes=true) = 0;
	virtual IWindowConsole* SetFontSize(int sz=0) = 0;
	virtual IWindowConsole* SetFont(LPCSTR name=NULL) = 0;

	virtual IWindowConsole* Clear() = 0;
	virtual IWindowConsole* SetCursor(int x, int y) = 0;
	virtual IWindowConsole* SetCursorToEnd() = 0;
	virtual IWindowConsole* SetCursorToTop() = 0;
	virtual IWindowConsole* Printf(LPCSTR fmt, ... ) = 0;
	virtual IWindowConsole* Printf(int level, LPCSTR fmt, ... ) = 0;
};

/*************************************************************************/ /**
 ** window container
 ** 
 **/ /*************************************************************************/ 
class IWindowContainer : public IWindow
{
public:
	virtual int GetNumItems() = 0;
	virtual IWindowContainer* AddItem(LPCSTR title, IWindow *lpWnd) = 0;
	//virtual IWindowContainer* AddItem(LPCSTR title, LPCSTR windowname) = 0;
	virtual IWindowContainer* RemoveItem(int Itemnum, IWindow **lpWnd=NULL) = 0; 
	virtual IWindowContainer* RemoveItem(IWindow *lpWnd) = 0;
	virtual IWindowContainer* SelectItem(int Itemnum) = 0;
};
/*************************************************************************/ /**
 ** window container with splitters
 ** 
 **/ /*************************************************************************/ 
class IWindowSplitter : public IWindowContainer
{
public:
	virtual int GetSplitterPos(int row) = 0;
	virtual IWindowSplitter* SetSplitterPos(int row, int y) = 0;
};
/*************************************************************************/ /**
 ** Scalar value DlgBox : title, val, slider
 ** 
 **/ /*************************************************************************/ 
class IControlScalar : public IWindow
{
public:
	virtual LPCSTR          GetValueAsString() = 0;
	virtual IControlScalar* SetIntMode(bool bYes=true) = 0;

	virtual IControlScalar* SetValue(float s) = 0;
	virtual float           GetValue() = 0;

    virtual IControlScalar* SetBounds(float min, float max) = 0;
	virtual IControlScalar* SetStep(float s) = 0;
    // When we drive a checkbox:
    virtual bool            GetChecked() = 0;
    virtual IControlScalar* SetChecked( bool bChecked ) = 0;
};
/*************************************************************************/ /**
 ** Vector 2,3,4 dimension
 ** 
 **/ /*************************************************************************/ 
class IControlVector : public IWindow
{
public:
	virtual IControlVector* SetIntMode(bool bYes=true) = 0;
	virtual IControlVector* SetDimension(int dim) = 0;
	virtual IControlVector* GetValuesAsInt(int *x, int *y=NULL, int *z=NULL, int *w=NULL) = 0;
	virtual IControlVector* GetValuesAsFloat(float *x, float *y=NULL, float *z=NULL, float *w=NULL) = 0;
	virtual IControlVector* GetValuesAsDouble(double *x, double *y=NULL, double *z=NULL, double *w=NULL) = 0;
	virtual IControlVector* SetValue(float x, float y, float z, float w) = 0;
	virtual IControlVector* SetValue(int n, float val) = 0;
};
/*************************************************************************/ /**
 ** Simple 1 line string + optional file browser button
 ** 
 **/ /*************************************************************************/ 
class IControlString : public IWindow
{
public:
	virtual IControlString* setString(LPCSTR str, bool bReplaceSelected=false) = 0;
	virtual IControlString* selectString(int start, int len) = 0;
	virtual IControlString* clearString(bool bOnlySelected=false) = 0;
	virtual LPCSTR getString() = 0;
	virtual LPCSTR getSelectedString() = 0;
	virtual int getStringLength() = 0;
	virtual LPCSTR ShowFileBrowseButton(LPCSTR filter=NULL,int bYes=1) = 0; ///<filter can be "Text|*.txt|All|*.*||"
};

/*************************************************************************/ /**
 ** Combo Box : to choose among various strings
 ** 
 **/ /*************************************************************************/ 
struct ComboBoxItem
{
    char strText[256];
    void*  pData;

    RECT  rcActive;
    bool  bVisible;
};

class IControlCombo : public IWindow
{
public:
    virtual IControlCombo*    RemoveAllItems() = 0;
    virtual IControlCombo*    RemoveItem( UINT index ) = 0;
    virtual IControlCombo*    SetDropHeight( UINT nHeight )  = 0;
    virtual IControlCombo*    SetScrollBarWidth( int nWidth )  = 0;
    virtual IControlCombo*    AddItem( const char* strText, void* pData ) = 0;

    virtual bool    ContainsItem( const char* strText, UINT iStart=0 ) = 0;
    virtual int     FindItem( const char* strText, UINT iStart=0 ) = 0;
    virtual void*   GetItemData( const char* strText ) = 0;
    virtual void*   GetItemData( int nIndex ) = 0;
    virtual int     GetScrollBarWidth() const  = 0;

    virtual int     GetSelectedIndex() const  = 0;
    virtual void*   GetSelectedData() = 0;
    virtual ComboBoxItem* GetSelectedItem() = 0;

    virtual UINT    GetNumItems() = 0;
    virtual ComboBoxItem* GetItem( UINT index ) = 0;

    virtual IControlCombo* SetSelectedByIndex( UINT index ) = 0;
    virtual IControlCombo* SetSelectedByText( const char* strText ) = 0;
    virtual IControlCombo* SetSelectedByData( void* pData ) = 0;  
};

/*************************************************************************/ /**
 **
 ** Toolbar
 ** 
 **/ /*************************************************************************/ 
class IControlToolbar : public IWindow
{
public:
	virtual IControlToolbar* addItem(LPCSTR title, LPCSTR imagefilename, int ddsidx=0, int states=1, unsigned long tag=0) = 0;
	virtual IControlToolbar* addItemFromMemoryBMP(LPCSTR title, void * imagedata, int states=1, unsigned long tag=0) = 0;
	virtual IControlToolbar* addItemFromMemoryPNG(LPCSTR title, void * imagedata, int states=1, unsigned long tag=0) = 0;
	virtual IControlToolbar* addItemFromMemoryTGA(LPCSTR title, void * imagedata, int states=1, unsigned long tag=0) = 0;
	virtual IControlToolbar* addItemFromMemoryDDS(LPCSTR title, void * imagedata, int ddsidx, int states=1, unsigned long tag=0) = 0;
	virtual IControlToolbar* InsertItemFromMemoryBMP(int pos, LPCSTR title, void * imagedata, int states=1, unsigned long tag=0) = 0;
	virtual IControlToolbar* InsertItemFromMemoryPNG(int pos, LPCSTR title, void * imagedata, int states=1, unsigned long tag=0) = 0;
	virtual IControlToolbar* InsertItemFromMemoryTGA(int pos, LPCSTR title, void * imagedata, int states=1, unsigned long tag=0) = 0;
	virtual IControlToolbar* InsertItemFromMemoryDDS(int pos, LPCSTR title, void * imagedata, int ddsidx=0, int states=1, unsigned long tag=0) = 0;
	virtual IControlToolbar* DeleteItem(int pos) = 0;
	virtual IControlToolbar* DeleteItem(LPCSTR title) = 0;
	virtual void ItemsSetState(int item, int state) = 0;
	virtual int ItemsGetState(int item) = 0;
	virtual IControlToolbar* Vertical(int bYes=-1) = 0;
	virtual IControlToolbar* ShowText(bool bYes=true) = 0;
	virtual IControlToolbar* SetMinMaxHeight(int hmin, int hmax) = 0;
	virtual IControlToolbar* GetItemInfos(int item, int &states, unsigned long &tag,
					char * title, int titlesz, 
					char * imagefname, int imagefnamesz,
					int &ddsidx) = 0;
	/// associate a python script to a button : pIPython is IPython class from nv_SvcScripting.h
	virtual IControlToolbar* SetPythonInterface(void * pIPython) = 0;
	virtual IControlToolbar* SetItemPythonScriptEvent(int item, LPCSTR script) = 0;
	virtual LPCSTR GetItemPythonScriptEvent(int item) = 0;
};

/*************************************************************************/ /**
 ** ListBox
 **/ /*************************************************************************/ 

struct ListBoxItem
{
    char strText[256];
    void*  pData;

    RECT  rcActive;
    bool  bSelected;
};

class IControlListBox : public IWindow
{
public:
    //virtual DWORD GetStyle() const { return m_dwStyle; }
    //virtual void SetStyle( DWORD dwStyle ) { m_dwStyle = dwStyle; }
    virtual int GetSize() const = 0;
    virtual int  GetScrollBarWidth() const = 0;
    virtual void SetScrollBarWidth( int nWidth ) = 0;
    virtual IControlListBox* SetBorder( int nBorder, int nMargin ) = 0;
    virtual IControlListBox* AddItem( const char *wszText, void *pData );
    virtual IControlListBox* InsertItem( int nIndex, const char *wszText, void *pData );
    virtual IControlListBox* RemoveItem( int nIndex );
    virtual IControlListBox* RemoveItemByText( char *wszText );
    virtual IControlListBox* RemoveItemByData( void *pData );
    virtual IControlListBox* RemoveAllItems();

    virtual ListBoxItem *GetItem( int nIndex );
    virtual int GetSelectedIndex( int nPreviousSelected = -1 );
    virtual ListBoxItem *GetSelectedItem( int nPreviousSelected = -1 ) = 0;
    virtual IControlListBox* SelectItem( int nNewIndex );
};

/*************************************************************************/ /**
 ** Singleton Interface of Windows message loop
 **/ /*************************************************************************/ 
class IWindowHandler : public ISmartRef
{
public:
    //
    // Get Interfaces of instanciated objects :
    //
    virtual IWindow*            Get(LPCSTR szID) = 0;
    virtual IWindowLog*         GetLog(LPCSTR szID) = 0;
    virtual IWindowConsole*     GetConsole(LPCSTR szID) = 0;
    virtual IWindowContainer*   GetContainer(LPCSTR szID) = 0;
    virtual IWindowSplitter*    GetSplitter(LPCSTR szID) = 0;
    virtual IControlVector*     GetVector(LPCSTR szID) = 0;
//QUESTION: Shall we change the Class (IControlScalar) for Checkbox ?
    virtual IControlScalar*     GetCheck(LPCSTR szID) = 0;
    virtual IControlScalar*     GetScalar(LPCSTR szID) = 0;
    virtual IControlString*     GetString(LPCSTR szID) = 0;
    virtual IControlToolbar*    GetToolbar(LPCSTR szID) = 0;
    virtual IControlVector*     GetColor(LPCSTR szID) = 0;
    virtual IControlCombo*      GetCombo(LPCSTR szID) = 0;
//QUESTION: Shall we change the Class (IControlScalar) for Button ctrl ?
    virtual IControlScalar*     GetButton(LPCSTR szID) = 0;
    virtual IControlCombo*      GetRadio(LPCSTR szID) = 0;
    //
    // Create Objects (handled by WindowHandler object) :
    //
    virtual IWindowLog*         CreateWindowLog(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IWindowConsole*     CreateWindowConsole(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IWindowContainer*   CreateWindowContainer(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IWindowContainer*   CreateWindowFolding(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IWindowSplitter*    CreateWindowSplitter(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IWindowContainer*   CreateWindowTab(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;

    virtual IControlVector*     CreateCtrlVector(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IControlScalar*     CreateCtrlCheck(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IControlScalar*     CreateCtrlScalar(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IControlString*     CreateCtrlString(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IControlToolbar*    CreateCtrlToolbar(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IControlVector*     CreateCtrlColor(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IControlCombo*      CreateCtrlCombo(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IWindow*            CreateCtrlButton(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IControlCombo*      CreateCtrlRadio(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    virtual IControlListBox*    CreateCtrlListBox(LPCSTR szID, LPCSTR title, IWindowContainer * parent=NULL) = 0;
    //
    // Events when no msg pump available
    //
	// return only when windows has finished.
	virtual unsigned int HandleMessageLoop_Blocking() = 0;
	// returns only wen a Key was pressed : Virtual Key code value returned
	virtual unsigned int HandleMessageLoop_ReturnOnKeyPressed() = 0;
	// returns At every event. Virtual Key Code returned. Or 0 otherwise.
	virtual unsigned int HandleMessageLoop_OnePass() = 0;
    //
	// returns 0 if the method handled the message
	//
	virtual bool MsftMsgProc( unsigned long hWnd, unsigned int uMsg, unsigned int wParam, unsigned long lParam ) = 0; ///< specific to Microsoft msg pump
    //
    // Glut event mgt
    //
    virtual void glutReshape(int w, int h) = 0;
    virtual void glutKeys(unsigned char c, int x, int y) = 0;
    virtual bool glutMouse(int button, int state, int x, int y) = 0;
    virtual bool glutMotion(int x, int y) = 0;
    virtual bool glutPassiveMotion(int x, int y) = 0;
    //
    // For DX9 or OGL
    // DX9 needs a device pointer. OGL doesn't : using the current context
    //
	virtual long CreateUIResources(LPCSTR windowName, void *devicePtr=NULL) = 0;
	virtual long ResetUIResources() = 0;   // OGL doesn't need it
	virtual void LostUIResources() = 0;    // OGL doesn't need it
	virtual void DestroyUIResources() = 0; // OGL doesn't need it
    //
    // Events for DX10 (following the names of DX10 DXUTgui )
    //
    virtual long OnD3D10CreateDevice(void *devitf) = 0;
    virtual long OnD3D10ResizedSwapChain( void *devitf, int w, int h ) = 0;
    virtual void OnD3D10ReleasingSwapChain() = 0;
    virtual void OnD3D10DestroyDevice() = 0;
    /** When rendering the UI is necessary
    */
    virtual void RenderUI(float tick) = 0;
    //
    // Events for all the windows/controls in this handler
    //
    virtual void Register(IEventsWnd *pDataBase) = 0;
	virtual void UnRegister(IEventsWnd *pClient=NULL) = 0;
    //
    // easy binding for variables (avoid managing them via events)
    //
    virtual bool VariableUnbind(void *ptr) = 0;
    virtual bool VariableFlush(void *ptr) = 0;
    virtual bool VariableBind(IWindow *pIWin, float *vec, int dim=1) = 0;
    virtual bool VariableBind(IWindow *pIWin, int *pVal, int dim=1) = 0;
    virtual bool VariableBind(IWindow *pIWin, unsigned int *pVal, int dim=1) = 0;
    virtual bool VariableBind(IWindow *pIWin, long *pVal, int dim=1) = 0;
    virtual bool VariableBind(IWindow *pIWin, unsigned long *pVal, int dim=1) = 0;
    virtual bool VariableBind(IWindow *pIWin, bool *pVal) = 0;
    virtual bool VariableBind(IWindow *pIWin, char *pVal) = 0;
    virtual bool VariableBind(IWindow *pIWin, char *string, int maxlen) = 0;
};
//
// Lib entry when used as a library
//
extern IWindowHandler* GetWndMsgHandler();
#endif

